#ifndef MAGPIE_LAYER_HPP
#define MAGPIE_LAYER_HPP

#include "server.hpp"
#include "surface.hpp"
#include "types.hpp"

#include <functional>
#include <memory>
#include <set>

#include "wlr-wrap-start.hpp"
#include <wlr/types/wlr_subcompositor.h>
#include "wlr-wrap-end.hpp"

class Layer final : public Surface {
  public:
	struct Listeners {
		std::reference_wrapper<Layer> parent;
		wl_listener map = {};
		wl_listener unmap = {};
		wl_listener destroy = {};
		wl_listener commit = {};
		wl_listener new_popup = {};
		wl_listener new_subsurface = {};
		explicit Listeners(Layer& parent) noexcept : parent(parent) {}
	};

  private:
	Listeners listeners;

  public:
	Server& server;
	Output& output;

	wlr_layer_surface_v1& wlr;
	wlr_scene_layer_surface_v1* scene_surface;
	magpie_scene_layer_t scene_layer = MAGPIE_SCENE_LAYER_NORMAL;

	Layer(Output& output, wlr_layer_surface_v1& surface) noexcept;
	~Layer() noexcept override;

	[[nodiscard]] wlr_surface* get_wlr_surface() const override;
	[[nodiscard]] Server& get_server() const override;
	[[nodiscard]] bool is_view() const override;
};

#endif
